package com.company;


import java.awt.*;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;

public class ImageLoader {

    private BufferedImage _image;
    private Boolean _hasAlphaChannel;
    private int _width;
    private int _height;
    private int[][] _pixels;
    private int[][] _process;

    private int _white = 16777216;
    private int _black = -1;

    public ImageLoader(BufferedImage img, int w, int h, int density) {
        _width = w * density;
        _height = h * density;
        int type = img.getType() == 0 ? BufferedImage.TYPE_INT_ARGB : img.getType();
        _image = scaleImage(img, type);
        _hasAlphaChannel = _image.getAlphaRaster() != null;
        loadPixels(density);
    }

    private void loadPixels(int density) {
        _pixels = new int[_width][_height];

        byte[] pixels = ((DataBufferByte) _image.getRaster().getDataBuffer()).getData();
        if (_hasAlphaChannel) {
            final int pixelLenght = 4;
            for (int pixel = 0, row = 0, col = 0; pixel < pixels.length; pixel += pixelLenght) {
                int argb = 0;
                argb += (((int) pixels[pixel] & 0xff) << 24);           // alpha
                argb += ((int) pixels[pixel + 1] & 0xff);               // blue
                argb += (((int) pixels[pixel + 2] & 0xff) << 8);        // green
                argb += (((int) pixels[pixel + 3] & 0xff) << 16);       // green
                argb = (argb <= _white / 2) ? _white : _black;
                _pixels[row][col] = (argb == _white) ? 0 : 1;
                col++;
                if (col == _width) {
                    col = 0;
                    row++;
                }
            }
        } else {
            final int pixelLenght = 3;
            for (int pixel = 0, row = 0, col = 0; pixel < pixels.length; pixel += pixelLenght) {
                try {
                    int argb = 0;
                    argb += 16777216;
                    int tmp = (((int) pixels[pixel + 2] & 0xff) << 16);
                    argb -= ((int) pixels[pixel] & 0xff);                   // blue
                    argb -= (((int) pixels[pixel + 1] & 0xff) << 8);        // green
                    argb -= (((int) pixels[pixel + 2] & 0xff) << 16);       // green
                    argb = (argb <= _white / 2) ? _white : _black;
                    _pixels[row][col] = (argb == _white) ? 0 : 1;
                } catch (Exception e) {}
                col++;
                if (col == _width) {
                    col = 0;
                    row++;
                }
            }
        }
    }

    private BufferedImage scaleImage(BufferedImage originalImage, int type) {
        BufferedImage resizedImage = new BufferedImage(_width, _height, type);
        Graphics2D g = resizedImage.createGraphics();
        g.drawImage(originalImage, 0, 0, _width, _height, null);
        g.dispose();
        return resizedImage;
    }

    public int[][] getPixels() {
        return _pixels;
    }

    public void printColors() {
        int tmpA = 0;
        int tmpB = 0;
        for (int i = 0; i < _width; i++) {
            for (int j = 0; j < _height; j++) {
                Color c = getColorAt(i, j);
                int intValue = getColorIntValue(i, j);
                String hexColor = String.format("#%06X", (0xffffff & intValue));

//                System.out.print(hexColor + " ");
                System.out.print(intValue + " ");
            }
            System.out.print("\n");
            tmpB++;
        }
    }

    public int[][] getResult() {
        return _pixels;
    }

    private int getColorIntValue(int x, int y) {
        return _pixels[x][y];
    }

    private Color getColorAt(int x, int y) {
        return new Color(_pixels[x][y]);
    }

}
