package com.company;

import java.text.Normalizer;
import java.util.ArrayList;
import java.util.HashMap;

public final class PolynomialGenerator {

    public static HashMap antiLogTable = initAntiLogTable();

    public static Formula generateForNCodeworks(int number) {
        if (antiLogTable == null) initAntiLogTable();

        Formula actual = new Formula();
        actual.addNumber(0, 1);
        actual.addNumber(0, 0);

        for (int i = 1; i < number; i++) {
            Formula next = initNext(i);
            actual = multiplyFormulasEachOther(actual, next);
        }

        return actual;
    }

    public static Formula initNext(int n) {
        Formula f = new Formula();
        f.addNumber(0, 1);
        f.addNumber(n, 0);
        return f;
    }

    public static Formula multiplyFormulasEachOther(Formula f1, Formula f2) {
        Formula newFormula = new Formula();
        for (int i = 0; i < f1.getSize(); i++) {
            PolNumber n1 = f1.getNumber(i);
            for (int j = 0; j < f2.getSize(); j++) {
                PolNumber n2 = f2.getNumber(j);
                newFormula.addNumber( getCorrectExponent(n1.a + n2.a), n1.x + n2.x);
            }
        }
        return combineSameX(newFormula);
    }

    public static int getCorrectExponent(int exponent) {
        if (exponent <= 255) return exponent;
        else {
            return (exponent % 256) + (int) Math.floor((double) exponent / 256);
        }
    }

    public static Formula combineSameX(Formula f) {
        int fSize = f.getSize();
        for (int i = 0; i < fSize; i++) {
            PolNumber number1 = f.getNumber(i);
            int x = number1.x;
            int a = number1.a;
            for (int j = 0; j < fSize; j++) {
                if (j == i) continue;
                PolNumber number2 = f.getNumber(j);
                if (number2.x == x) {
                    PolNumber n = new PolNumber();
                    int tmp = getNumberForExponent(a) ^ getNumberForExponent(number2.a);
                    n.a = getExponentForNumber(tmp);
                    n.x = x;

                    f.remove(j);
                    f.remove(i);
                    f.addNumber(i, n);
                    fSize--;
                    i = -1;
                    break;
                }
            }
        }
//        System.out.println(f.toString());

        return f;
    }

    public static int getExponentForNumber(int number) {
        if (antiLogTable == null) initAntiLogTable();
        for (int i = 0; i < antiLogTable.size(); i++) {
            if ((Integer) antiLogTable.get(i) == number) return i;
        }
        return 0;
    }

    public static int getNumberForExponent(int exponent) {
        return (Integer) antiLogTable.get(exponent);
    }

    public static void main(String[] args) {
        generateForNCodeworks(10);
    }

    private static HashMap initAntiLogTable() {
        HashMap map = new HashMap();

map.put(0,1);
map.put(1,2);

map.put(2,4);

map.put(3,8);

map.put(4,16);

map.put(5,32);

map.put(6,64);

map.put(7,128);

map.put(8,29);

map.put(9,58);

map.put(10,116);

map.put(11,232);

map.put(12,205);

map.put(13,135);

map.put(14,19);

map.put(15,38);

map.put(16,76);

map.put(17,152);

map.put(18,45);

map.put(19,90);

map.put(20,180);

map.put(21,117);

map.put(22,234);

map.put(23,201);

map.put(24,143);

map.put(25,3);

map.put(26,6);

map.put(27,12);

map.put(28,24);

map.put(29,48);

map.put(30,96);

map.put(31,192);

map.put(32,157);

map.put(33,39);

map.put(34,78);

map.put(35,156);

map.put(36,37);

map.put(37,74);

map.put(38,148);

map.put(39,53);

map.put(40,106);

map.put(41,212);

map.put(42,181);

map.put(43,119);

map.put(44,238);

map.put(45,193);

map.put(46,159);

map.put(47,35);

map.put(48,70);

map.put(49,140);

map.put(50,5);

map.put(51,10);

map.put(52,20);

map.put(53,40);

map.put(54,80);

map.put(55,160);

map.put(56,93);

map.put(57,186);

map.put(58,105);

map.put(59,210);

map.put(60,185);

map.put(61,111);

map.put(62,222);

map.put(63,161);

map.put(64,95);

map.put(65,190);

map.put(66,97);

map.put(67,194);

map.put(68,153);

map.put(69,47);

map.put(70,94);

map.put(71,188);

map.put(72,101);

map.put(73,202);

map.put(74,137);

map.put(75,15);

map.put(76,30);

map.put(77,60);

map.put(78,120);

map.put(79,240);

map.put(80,253);

map.put(81,231);

map.put(82,211);

map.put(83,187);

map.put(84,107);

map.put(85,214);

map.put(86,177);

map.put(87,127);

map.put(88,254);

map.put(89,225);

map.put(90,223);

map.put(91,163);

map.put(92,91);

map.put(93,182);

map.put(94,113);

map.put(95,226);

map.put(96,217);

map.put(97,175);

map.put(98,67);

map.put(99,134);

map.put(100,17);

map.put(101,34);

map.put(102,68);

map.put(103,136);

map.put(104,13);

map.put(105,26);

map.put(106,52);

map.put(107,104);

map.put(108,208);

map.put(109,189);

map.put(110,103);

map.put(111,206);

map.put(112,129);

map.put(113,31);

map.put(114,62);

map.put(115,124);

map.put(116,248);

map.put(117,237);

map.put(118,199);

map.put(119,147);

map.put(120,59);

map.put(121,118);

map.put(122,236);

map.put(123,197);

map.put(124,151);

map.put(125,51);

map.put(126,102);

map.put(127,204);

map.put(128,133);

map.put(129,23);

map.put(130,46);

map.put(131,92);

map.put(132,184);

map.put(133,109);

map.put(134,218);

map.put(135,169);

map.put(136,79);

map.put(137,158);

map.put(138,33);

map.put(139,66);

map.put(140,132);

map.put(141,21);

map.put(142,42);

map.put(143,84);

map.put(144,168);

map.put(145,77);

map.put(146,154);

map.put(147,41);

map.put(148,82);

map.put(149,164);

map.put(150,85);

map.put(151,170);

map.put(152,73);

map.put(153,146);

map.put(154,57);

map.put(155,114);

map.put(156,228);

map.put(157,213);

map.put(158,183);

map.put(159,115);

map.put(160,230);

map.put(161,209);

map.put(162,191);

map.put(163,99);

map.put(164,198);

map.put(165,145);

map.put(166,63);

map.put(167,126);

map.put(168,252);

map.put(169,229);

map.put(170,215);

map.put(171,179);

map.put(172,123);

map.put(173,246);

map.put(174,241);

map.put(175,255);

map.put(176,227);

map.put(177,219);

map.put(178,171);

map.put(179,75);

map.put(180,150);

map.put(181,49);

map.put(182,98);

map.put(183,196);

map.put(184,149);

map.put(185,55);

map.put(186,110);

map.put(187,220);

map.put(188,165);

map.put(189,87);

map.put(190,174);

map.put(191,65);

map.put(192,130);

map.put(193,25);

map.put(194,50);

map.put(195,100);

map.put(196,200);

map.put(197,141);

map.put(198,7);

map.put(199,14);

map.put(200,28);

map.put(201,56);

map.put(202,112);

map.put(203,224);

map.put(204,221);

map.put(205,167);

map.put(206,83);

map.put(207,166);

map.put(208,81);

map.put(209,162);

map.put(210,89);

map.put(211,178);

map.put(212,121);

map.put(213,242);

map.put(214,249);

map.put(215,239);

map.put(216,195);

map.put(217,155);

map.put(218,43);

map.put(219,86);

map.put(220,172);

map.put(221,69);

map.put(222,138);

map.put(223,9);

map.put(224,18);

map.put(225,36);

map.put(226,72);

map.put(227,144);

map.put(228,61);

map.put(229,122);

map.put(230,244);

map.put(231,245);

map.put(232,247);

map.put(233,243);

map.put(234,251);

map.put(235,235);

map.put(236,203);

map.put(237,139);

map.put(238,11);

map.put(239,22);

map.put(240,44);

map.put(241,88);

map.put(242,176);

map.put(243,125);

map.put(244,250);

map.put(245,233);

map.put(246,207);

map.put(247,131);

map.put(248,27);

map.put(249,54);

map.put(250,108);

map.put(251,216);

map.put(252,173);

map.put(253,71);

map.put(254,142);

map.put(255,1);

        return map;
    }
}
