package com.company.QR;

import com.company.QRCodeClient;

import java.util.HashMap;

public class QRCode implements QRCodeClient {

    private HashMap alphaNumericTable;
    private QRCodeErrorCorrectionLevel correctionLevel;
    private QRCodeMatrix matrix;
    private QRCodeVersion version;
    private QRTextCoder textCoder;
    private QRCodeMode mode;

    private String message;

    public QRCode() {
        initTable();
    }

    public void encode(String msg, String lvl) {
        correctionLevel = new QRCodeErrorCorrectionLevel(lvl, this);
        message = msg;

        mode = new QRCodeMode(this);
        version = new QRCodeVersion(this);

        int size = this.version.getSizeForVersion();
        textCoder = new QRTextCoder(this);

        int[] errorCorrection = correctionLevel.errorCorrectionCoding();

        //printDebug();
        matrix = new QRCodeMatrix(size, size,this, errorCorrection);
//        matrix.addVersionData(version.generateVersionData(matrix.getUsedMask()));
        matrix.updateColors();

//        System.out.println("--------------RESULT--------------");
//        matrix.printDebugMatrix();
//        System.out.println("----------------------------------");
    }

    public void printDebug() {
        System.out.println("--------INFO-------------");
        System.out.println("Correction level string: " + this.correctionLevel.getStringCorrectionLevel());
        System.out.println("Correction level int:    " + this.correctionLevel.getIntCorrectinLevel());
        System.out.println("Message:        " + getMsg());
        System.out.println("Message lenght: " + getMsg().length());
        System.out.println("Mode:    " + this.mode.getByteMode());
        System.out.println("Version: " + this.version.getVersion());
        System.out.println("-------------------------");
    }

    public int getTotalCapability() {
//        return this.version.getCapability(this.version.getVersion(), this.correctionLevel.getIntCorrectinLevel());
        return this.correctionLevel.getCapabilityForModeAndVersion(
            this.correctionLevel.getCorrectionString());
    }

    public String getBytes() { return this.textCoder.getBytes(); }

    public int[] getAlignmentPositionByVersion() { return this.version.getAlignmentPositionByVersion(); }

    public int getTotalCapability(String id) {
        return this.correctionLevel.getCapabilityForModeAndVersion(id);
    }

    public void setVersion(QRCodeVersion v) {
        this.version = v;
    }

    public QRCodeVersion getVersion() {
        return this.version;
    }

    public void setMatrix(QRCodeMatrix m) {
        this.matrix = m;
    }

    public QRCodeMatrix getMatrix() {
        return this.matrix;
    }

    public byte getMode() { return this.mode.getMode(); }

    public String getByteMode() { return this.mode.getByteMode(); }

    public Boolean hasAlphaNumericTableChar(char c) {
        return this.alphaNumericTable.containsKey(c);
    }

    public HashMap getAlphaNumericTable() {
        return this.alphaNumericTable;
    }

    public QRCodeErrorCorrectionLevel getCorrectionLevel() {
        return this.correctionLevel;
    }

    public String getMsg() {
        return this.message;
    }

    public int getIntCorrectionlevel() { return correctionLevel.getIntCorrectinLevel(); }

    public String getStringCorrectionLevel() { return correctionLevel.getStringCorrectionLevel(); }

    private void initTable() {
        alphaNumericTable = new HashMap();
        alphaNumericTable.put('0', 0);
        alphaNumericTable.put('1', 1);
        alphaNumericTable.put('2', 2);
        alphaNumericTable.put('3', 3);
        alphaNumericTable.put('4', 4);
        alphaNumericTable.put('5', 5);
        alphaNumericTable.put('6', 6);
        alphaNumericTable.put('7', 7);
        alphaNumericTable.put('8', 8);
        alphaNumericTable.put('9', 9);
        alphaNumericTable.put('A', 10);
        alphaNumericTable.put('B', 11);
        alphaNumericTable.put('C', 12);
        alphaNumericTable.put('D', 13);
        alphaNumericTable.put('E', 14);
        alphaNumericTable.put('F', 15);
        alphaNumericTable.put('G', 16);
        alphaNumericTable.put('H', 17);
        alphaNumericTable.put('I', 18);
        alphaNumericTable.put('J', 19);
        alphaNumericTable.put('K', 20);
        alphaNumericTable.put('L', 21);
        alphaNumericTable.put('M', 22);
        alphaNumericTable.put('N', 23);
        alphaNumericTable.put('O', 24);
        alphaNumericTable.put('P', 25);
        alphaNumericTable.put('Q', 26);
        alphaNumericTable.put('R', 27);
        alphaNumericTable.put('S', 28);
        alphaNumericTable.put('T', 29);
        alphaNumericTable.put('U', 30);
        alphaNumericTable.put('V', 31);
        alphaNumericTable.put('W', 32);
        alphaNumericTable.put('X', 33);
        alphaNumericTable.put('Y', 34);
        alphaNumericTable.put('Z', 35);
        alphaNumericTable.put(' ', 36);
        alphaNumericTable.put('$', 37);
        alphaNumericTable.put('%', 38);
        alphaNumericTable.put('*', 39);
        alphaNumericTable.put('+', 40);
        alphaNumericTable.put('-', 41);
        alphaNumericTable.put('.', 42);
        alphaNumericTable.put('/', 43);
        alphaNumericTable.put(':', 44);
    }

}
