package com.company.QR;

import com.company.Formula;
import com.company.PolNumber;
import com.company.PolynomialGenerator;
import com.company.QRCodeClient;

import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashMap;

public class QRCodeErrorCorrectionLevel {
    private int level;
    private String levelString;
    private QRCodeClient client;
    private HashMap dataTable;
    private String allVersion;

    public QRCodeErrorCorrectionLevel(String lvl, QRCode c) {
        levelString = lvl;
        client = c;
        if (lvl.equals("L")) {
            level = 0;
        } else if (lvl.equals("M")) {
            level = 1;
        } else if (lvl.equals("Q")) {
            level = 2;
        } else if (lvl.equals("H")) {
            level = 3;
        }
        initCorrectionCapabilites();
    }

    public int getIntCorrectinLevel() {
        return level;
    }

    public int getCapabilityForModeAndVersion(String id) {
        Block b = (Block) dataTable.get(id);
        return b.getTotalDataCodeWorks();
    }

    public int getCapacityInforFormodeAndVersion(int info, String id) {
        Block b = (Block) dataTable.get(id);
        return b.data[info];
    }

    public int[] errorCorrectionCoding() {
        String bytes = client.getBytes();
        Formula mesNumbers = getNumbersForPol(bytes);
        Formula polNumbers = PolynomialGenerator.generateForNCodeworks(
                getCapacityInforFormodeAndVersion(1, getCorrectionString()));
//        System.out.println("\nPolNumbers: n-for " + getCapacityInforFormodeAndVersion(1, getCorrectionString()));
        Block b = (Block) dataTable.get(getCorrectionString());
        int steps = b.getTotalDataCodeWorks();
//        System.out.println("PolDivision steps: " + steps);

        int dif = mesNumbers.getHighestX() - polNumbers.getHighestX();
        polNumbers.increaseX(dif);

        Formula result = divideMessagePol(mesNumbers, polNumbers, steps);
        result.convertToAlphaAnotation();
//        System.out.println("\n result \n" + result.toString());
        return result.getNumbers();
    }

    public Formula divideMessagePol(Formula mesNumbers, Formula polNumbers, int steps) {
        mesNumbers.toString();
        if (steps == 0) {
            return mesNumbers;
        }
        mesNumbers.convertToAlphaAnotation();

        Formula pol = polNumbers.clone();
        int increasingPolNumbers = PolynomialGenerator.getExponentForNumber(mesNumbers.getPolNumberOfHighestX().number);
        if (increasingPolNumbers == 0) {
            pol.setNumbers(0);
            pol.convertToAlphaAnotation();
        } else {
            pol.increaseA(increasingPolNumbers);
            pol.convertToIntegerAnotation();
        }

        mesNumbers.xorFormulas(pol);
        mesNumbers.remove(0);
        polNumbers.decreaseX(1);

        steps--;

        return divideMessagePol(mesNumbers, polNumbers, steps);
    }

    public Formula getNumbersForPol(String bytes) {
        Formula formula = new Formula();
        ArrayList<Integer> numbers = new ArrayList<Integer>();
        for (int i = 0; i < bytes.length(); i += 8) {
            String number = bytes.substring(i, i + 8);
            System.out.print("," + Integer.parseInt(number, 2));
            PolNumber n = new PolNumber();
            n.number = Integer.parseInt(number, 2);
            formula.addNumber(n);
        }
        int increaseX = getCapacityInforFormodeAndVersion(0, getCorrectionString());
        int maxSize = formula.getSize();
        for (int i = 0; i < maxSize; i++) {
            PolNumber n = formula.getNumber(i);
            n.x = maxSize - i - 1;
            n.increaseX(increaseX);
        }
        return formula;
    }

    public Formula getFormulaFromNumbers(ArrayList<Integer> numbers) {
        Formula formula = new Formula();
        int increaseX = getCapacityInforFormodeAndVersion(1, getCorrectionString());
        int maxSize = numbers.size();
        for (int i = 0; i < maxSize; i++) {
            PolNumber n = new PolNumber();
            n.x = maxSize - i - 1;
            n.increaseX(increaseX);
            n.a = PolynomialGenerator.getExponentForNumber(numbers.get(i));
            formula.addNumber(n);
        }
        return formula;
    }

    public String getCorrectionString() {
        return client.getVersion().getVersion() + "-" + getStringCorrectionLevel();
    }

    public void initCorrectionCapabilites() {
        dataTable = new HashMap();
        dataTable.put("1-L", new Block(19, 7, 1, 19, 0,0));
        dataTable.put("1-M", new Block(16, 10, 1, 16, 0,0));
        dataTable.put("1-Q", new Block(13, 13, 1, 13, 0,0));
        dataTable.put("1-H", new Block(9, 17, 1, 9, 0,0));
        dataTable.put("2-L", new Block(34, 10, 1, 34, 0,0));
        dataTable.put("2-M", new Block(28, 16, 1, 28, 0,0));
        dataTable.put("2-Q", new Block(22, 22, 1, 22, 0,0));
        dataTable.put("2-H", new Block(16, 28, 1, 16, 0,0));
        dataTable.put("3-L", new Block(55, 15, 1, 55, 0,0));
        dataTable.put("3-M", new Block(44, 26, 1, 44, 0,0));
        dataTable.put("3-Q", new Block(34, 18, 1, 19, 0,0));
        dataTable.put("3-H", new Block(26, 22, 1, 19, 0,0));
        dataTable.put("4-L", new Block(80, 20, 1, 19, 0,0));
        dataTable.put("4-M", new Block(64, 18, 1, 19, 0,0));
        dataTable.put("4-Q", new Block(48, 26, 1, 19, 0,0));
        dataTable.put("4-H", new Block(36, 16, 1, 19, 0,0));
        dataTable.put("5-L", new Block(108, 26, 1, 19, 0,0));
        dataTable.put("5-M", new Block(86, 24, 1, 19, 0,0));
        dataTable.put("5-Q", new Block(62, 18, 1, 19, 0,0));
        dataTable.put("5-H", new Block(46, 22, 1, 19, 0,0));
        dataTable.put("6-L", new Block(136, 18, 1, 19, 0,0));
        dataTable.put("6-M", new Block(108, 16, 1, 19, 0,0));
        dataTable.put("6-Q", new Block(76, 24, 1, 19, 0,0));
        dataTable.put("6-H", new Block(60, 28, 1, 19, 0,0));
        dataTable.put("7-L", new Block(156, 20, 1, 19, 0,0));
        dataTable.put("7-M", new Block(124, 18, 1, 19, 0,0));
        dataTable.put("7-Q", new Block(88, 18, 1, 19, 0,0));
        dataTable.put("7-H", new Block(66, 26, 1, 19, 0,0));
        dataTable.put("8-L", new Block(194, 24, 1, 19, 0,0));
        dataTable.put("8-M", new Block(154, 22, 1, 19, 0,0));
        dataTable.put("8-Q", new Block(110, 22, 1, 19, 0,0));
        dataTable.put("8-H", new Block(86, 26, 1, 19, 0,0));
        dataTable.put("9-L", new Block(232, 30, 1, 19, 0,0));
        dataTable.put("9-M", new Block(182, 22, 1, 19, 0,0));
        dataTable.put("9-Q", new Block(132, 20, 1, 19, 0,0));
        dataTable.put("9-H", new Block(100, 24, 1, 19, 0,0));
        dataTable.put("10-L", new Block(274, 18, 1, 19, 0,0));
        dataTable.put("10-M", new Block(216, 26, 1, 19, 0,0));
        dataTable.put("10-Q", new Block(154, 24, 1, 19, 0,0));
        dataTable.put("10-H", new Block(122, 28, 1, 19, 0,0));
        dataTable.put("11-L", new Block(324, 20, 1, 19, 0,0));
        dataTable.put("11-M", new Block(254, 30, 1, 19, 0,0));
        dataTable.put("11-Q", new Block(180, 28, 1, 19, 0,0));
        dataTable.put("11-H", new Block(140, 24, 1, 19, 0,0));
        dataTable.put("12-L", new Block(370, 24, 1, 19, 0,0));
        dataTable.put("12-M", new Block(290, 22, 1, 19, 0,0));
        dataTable.put("12-Q", new Block(206, 26, 1, 19, 0,0));
        dataTable.put("12-H", new Block(158, 28, 1, 19, 0,0));
        dataTable.put("13-L", new Block(428, 26, 1, 19, 0,0));
        dataTable.put("13-M", new Block(334, 22, 1, 19, 0,0));
        dataTable.put("13-Q", new Block(244, 24, 1, 19, 0,0));
        dataTable.put("13-H", new Block(180, 22, 1, 19, 0,0));
        dataTable.put("14-L", new Block(461, 30, 1, 19, 0,0));
        dataTable.put("14-M", new Block(365, 24, 1, 19, 0,0));
        dataTable.put("14-Q", new Block(261, 20, 1, 19, 0,0));
        dataTable.put("14-H", new Block(197, 24, 1, 19, 0,0));
        dataTable.put("15-L", new Block(523, 22, 1, 19, 0,0));
        dataTable.put("15-M", new Block(415, 24, 1, 19, 0,0));
        dataTable.put("15-Q", new Block(295, 30, 1, 19, 0,0));
        dataTable.put("15-H", new Block(223, 24, 1, 19, 0,0));
        dataTable.put("16-L", new Block(589, 24, 1, 19, 0,0));
        dataTable.put("16-M", new Block(453, 28, 1, 19, 0,0));
        dataTable.put("16-Q", new Block(325, 24, 1, 19, 0,0));
        dataTable.put("16-H", new Block(253, 30, 1, 19, 0,0));
        dataTable.put("17-L", new Block(647, 28, 1, 19, 0,0));
        dataTable.put("17-M", new Block(507, 28, 1, 19, 0,0));
        dataTable.put("17-Q", new Block(367, 28, 1, 19, 0,0));
        dataTable.put("17-H", new Block(283, 28, 1, 19, 0,0));
        dataTable.put("18-L", new Block(721, 30, 1, 19, 0,0));
        dataTable.put("18-M", new Block(563, 26, 1, 19, 0,0));
        dataTable.put("18-Q", new Block(397, 28, 1, 19, 0,0));
        dataTable.put("18-H", new Block(313, 28, 1, 19, 0,0));
        dataTable.put("19-L", new Block(795, 28, 1, 19, 0,0));
        dataTable.put("19-M", new Block(627, 26, 1, 19, 0,0));
        dataTable.put("19-Q", new Block(445, 26, 1, 19, 0,0));
        dataTable.put("19-H", new Block(341, 26, 1, 19, 0,0));
        dataTable.put("20-L", new Block(861, 28, 1, 19, 0,0));
        dataTable.put("20-M", new Block(669, 26, 1, 19, 0,0));
        dataTable.put("20-Q", new Block(485, 30, 1, 19, 0,0));
        dataTable.put("20-H", new Block(385, 28, 1, 19, 0,0));
        dataTable.put("21-L", new Block(932, 28, 1, 19, 0,0));
        dataTable.put("21-M", new Block(714, 26, 1, 19, 0,0));
        dataTable.put("21-Q", new Block(512, 28, 1, 19, 0,0));
        dataTable.put("21-H", new Block(406, 30, 1, 19, 0,0));
        dataTable.put("22-L", new Block(1006, 28, 1, 19, 0,0));
        dataTable.put("22-M", new Block(762, 28, 1, 19, 0,0));
        dataTable.put("22-Q", new Block(568, 30, 1, 19, 0,0));
        dataTable.put("22-H", new Block(442, 24, 1, 19, 0,0));
        dataTable.put("23-L", new Block(1094, 30, 1, 19, 0,0));
        dataTable.put("23-M", new Block(860, 28, 1, 19, 0,0));
        dataTable.put("23-Q", new Block(614, 30, 1, 19, 0,0));
        dataTable.put("23-H", new Block(464, 30, 1, 19, 0,0));
        dataTable.put("24-L", new Block(1174, 30, 1, 19, 0,0));
        dataTable.put("24-M", new Block(914, 28, 1, 19, 0,0));
        dataTable.put("24-Q", new Block(664, 30, 1, 19, 0,0));
        dataTable.put("24-H", new Block(514, 30, 1, 19, 0,0));
        dataTable.put("25-L", new Block(1276, 26, 1, 19, 0,0));
        dataTable.put("25-M", new Block(1000, 28, 1, 19, 0,0));
        dataTable.put("25-Q", new Block(718, 30, 1, 19, 0,0));
        dataTable.put("25-H", new Block(538, 30, 1, 19, 0,0));
        dataTable.put("26-L", new Block(1370, 28, 1, 19, 0,0));
        dataTable.put("26-M", new Block(1062, 28, 1, 19, 0,0));
        dataTable.put("26-Q", new Block(754, 28, 1, 19, 0,0));
        dataTable.put("26-H", new Block(596, 30, 1, 19, 0,0));
        dataTable.put("27-L", new Block(1468, 30, 1, 19, 0,0));
        dataTable.put("27-M", new Block(1128, 28, 1, 19, 0,0));
        dataTable.put("27-Q", new Block(808, 0000000000, 1, 19, 0,0));
        dataTable.put("27-H", new Block(628, 28, 1, 19, 0,0));
        dataTable.put("28-L", new Block(1531, 28, 1, 19, 0,0));
        dataTable.put("28-M", new Block(1193, 30, 1, 19, 0,0));
        dataTable.put("28-Q", new Block(871, 30, 1, 19, 0,0));
        dataTable.put("28-H", new Block(661, 30, 1, 19, 0,0));
        dataTable.put("29-L", new Block(1631, 28, 1, 19, 0,0));
        dataTable.put("29-M", new Block(1267, 30, 1, 19, 0,0));
        dataTable.put("29-Q", new Block(911, 30, 1, 19, 0,0));
        dataTable.put("29-H", new Block(701, 30, 1, 19, 0,0));
        dataTable.put("30-L", new Block(1735, 28, 1, 19, 0,0));
        dataTable.put("30-M", new Block(1373, 30, 1, 19, 0,0));
        dataTable.put("30-Q", new Block(985, 30, 1, 19, 0,0));
        dataTable.put("30-H", new Block(745, 30, 1, 19, 0,0));
        dataTable.put("31-L", new Block(1843, 7, 1, 19, 0,0));
        dataTable.put("31-M", new Block(1455, 7, 1, 19, 0,0));
        dataTable.put("31-Q", new Block(1033, 7, 1, 19, 0,0));
        dataTable.put("31-H", new Block(793, 7, 1, 19, 0,0));
        dataTable.put("32-L", new Block(1955, 7, 1, 19, 0,0));
        dataTable.put("32-M", new Block(1541, 7, 1, 19, 0,0));
        dataTable.put("32-Q", new Block(1115, 7, 1, 19, 0,0));
        dataTable.put("32-H", new Block(845, 7, 1, 19, 0,0));
        dataTable.put("33-L", new Block(2071, 7, 1, 19, 0,0));
        dataTable.put("33-M", new Block(1631, 7, 1, 19, 0,0));
        dataTable.put("33-Q", new Block(1171, 7, 1, 19, 0,0));
        dataTable.put("33-H", new Block(901, 7, 1, 19, 0,0));
        dataTable.put("34-L", new Block(2191, 7, 1, 19, 0,0));
        dataTable.put("34-M", new Block(1725, 7, 1, 19, 0,0));
        dataTable.put("34-Q", new Block(1231, 7, 1, 19, 0,0));
        dataTable.put("34-H", new Block(961, 7, 1, 19, 0,0));
        dataTable.put("35-L", new Block(2306, 7, 1, 19, 0,0));
        dataTable.put("35-M", new Block(1812, 7, 1, 19, 0,0));
        dataTable.put("35-Q", new Block(1286, 7, 1, 19, 0,0));
        dataTable.put("35-H", new Block(986, 7, 1, 19, 0,0));
        dataTable.put("36-L", new Block(2434, 7, 1, 19, 0,0));
        dataTable.put("36-M", new Block(1914, 7, 1, 19, 0,0));
        dataTable.put("36-Q", new Block(1354, 7, 1, 19, 0,0));
        dataTable.put("36-H", new Block(1054, 7, 1, 19, 0,0));
        dataTable.put("37-L", new Block(2566, 7, 1, 19, 0,0));
        dataTable.put("37-M", new Block(1992, 7, 1, 19, 0,0));
        dataTable.put("37-Q", new Block(1426, 7, 1, 19, 0,0));
        dataTable.put("37-H", new Block(1096, 7, 1, 19, 0,0));
        dataTable.put("38-L", new Block(2702, 7, 1, 19, 0,0));
        dataTable.put("38-M", new Block(2102, 7, 1, 19, 0,0));
        dataTable.put("38-Q", new Block(1502, 7, 1, 19, 0,0));
        dataTable.put("38-H", new Block(1142, 7, 1, 19, 0,0));
        dataTable.put("39-L", new Block(2812, 7, 1, 19, 0,0));
        dataTable.put("39-M", new Block(2216, 7, 1, 19, 0,0));
        dataTable.put("39-Q", new Block(1582, 7, 1, 19, 0,0));
        dataTable.put("39-H", new Block(1222, 7, 1, 19, 0,0));
        dataTable.put("40-L", new Block(2956, 7, 1, 19, 0,0));
        dataTable.put("40-M", new Block(2334, 7, 1, 19, 0,0));
        dataTable.put("40-Q", new Block(1666, 7, 1, 19, 0,0));
        dataTable.put("40-H", new Block(1276, 7, 1, 19, 0,0));

    }

    public String getStringCorrectionLevel() {
        switch (level) {
            case 0: return "L";
            case 1: return "Q";
            case 2: return "M";
            case 3: return "H";
        }
        return null;
    }

    class Block {
        int[] data;

        public Block(int dataForVersionEC,
                     int ECCodePerBlock,
                     int nOfBlocks1,
                     int nOfGroupblocks1,
                     int nOfBlocks2,
                     int nOfGroupblocks2) {
            data = new int[6];
            data[0] = dataForVersionEC;
            data[1] = ECCodePerBlock;
            data[2] = nOfBlocks1;
            data[3] = nOfGroupblocks1;
            data[4] = nOfBlocks2;
            data[5] = nOfGroupblocks2;
        }

        public int getTotalDataCodeWorks() {
            return data[2] * data[3] + data[4] * data[5];
        }

        public int getInfo(int id) {
            return data[id];
        }

        public int getCapability() {
            return data[0];
        }
    }
}
