package com.company.QR;

import com.company.Masks;
import com.company.QRCodeClient;

import java.util.Arrays;

public class QRCodeMatrix {

    private QRCodeClient client;
    private int width;
    private int height;
    private int[][] matrix;
    private int[] correctionBits;
    private int maskUsed;
    private String bytes;

    private final int[][] finderPattern = new int[][]{
            { 5, 5, 5, 5, 5, 5, 5 },
            { 5, 4, 4, 4, 4, 4, 5 },
            { 5, 4, 5, 5, 5, 4, 5 },
            { 5, 4, 5, 5, 5, 4, 5 },
            { 5, 4, 5, 5, 5, 4, 5 },
            { 5, 4, 4, 4, 4, 4, 5 },
            { 5, 5, 5, 5, 5, 5, 5 }
    };

    private final int[][] alignmentPattern = new int[][]{
            { 7, 7, 7, 7, 7 },
            { 7, 6, 6, 6, 7 },
            { 7, 6, 7, 6, 7 },
            { 7, 6, 6, 6, 7 },
            { 7, 7, 7, 7, 7 }
    };

    public QRCodeMatrix(int w, int h, QRCodeClient c, int[] b) {
        client = c;
        width = w;
        height = h;

        correctionBits = b;

        matrix = new int[w][h];
        for (int[] row : matrix) {
            Arrays.fill(row, -1);
        }

        setPattern();
//        printDebugMatrix();
        placeBits();

//        int[][] n = getBestMatrixPenalty();
//        matrix = n;
        matrix = getBestMatrixPenalty();

//        System.out.println("----------------");
//        System.out.println("new Mask: " + maskUsed);
//        printDebugMatrix();
    }

    public void addVersionData(int[] data) {
        if (client.getVersion().version >= 7) {

        } else {
            matrix[0][8] = data[14];
            matrix[1][8] = data[13];
            matrix[2][8] = data[12];
            matrix[3][8] = data[11];
            matrix[4][8] = data[10];
            matrix[5][8] = data[9];

            matrix[7][8] = data[8];
            matrix[8][8] = data[7];
            matrix[8][7] = data[6];

            matrix[8][5] = data[5];
            matrix[8][4] = data[4];
            matrix[8][3] = data[3];
            matrix[8][2] = data[2];
            matrix[8][1] = data[1];
            matrix[8][0] = data[0];


            matrix[8][width - 1] = data[14];
            matrix[8][width - 2] = data[13];
            matrix[8][width - 3] = data[12];
            matrix[8][width - 4] = data[11];
            matrix[8][width - 5] = data[10];
            matrix[8][width - 6] = data[9];
            matrix[8][width - 7] = data[8];
            matrix[8][width - 8] = data[7];

            matrix[width - 7][8] = data[6];
            matrix[width - 6][8] = data[5];
            matrix[width - 5][8] = data[4];
            matrix[width - 4][8] = data[3];
            matrix[width - 3][8] = data[2];
            matrix[width - 2][8] = data[1];
            matrix[width - 1][8] = data[0];
        }
    }

    public void updateColors() {
        for (int y = 0; y < width; y++) {
            for (int x = 0; x < width; x++) {
                matrix[y][x] = (matrix[y][x] % 2 == 0) ? 0 : 1;
            }
        }
    }

    public int[][] getMatrix() {
        return matrix;
    }

    public int getUsedMask() {
        return maskUsed;
    }

    private int[][] getBestMatrixPenalty() {
        Masks.setClient(client);

//        System.out.println("before mask: ");
//        printDebugMatrix();
//
//        System.out.println("----------------");

        /*int pokus = 7;
//        matrix = Masks.applyMaskForMatrix(pokus, matrix);
        int[][] o = Masks.applyMaskForMatrix(pokus, matrix);
        maskUsed = pokus;
        int p = Masks.countPenaltyForMatrix(o);
        System.out.println("Mask " + pokus + " penalty: " + p);
        return o;*/

        int lowestP = 0;
        int from = 0;
        int to = 8;
        int[][] best = new int[width][width];
        for (int i = from; i < to; i++) {
            int[][] m = Masks.applyMaskForMatrix(i, matrix.clone());
            int p = Masks.countPenaltyForMatrix(m);
//            System.out.println("Mask " + i + " penalty: " + p);
//            printDebugMatrix(m);
            if (i == from || p < lowestP) {
                maskUsed = i;
                lowestP = p;
                best = m.clone();
            }
        }
        return best;
    }

    private void setPattern() {
        addFinderPatterns();
        addSeparators();
        addAlignmentPatterns();
        addTimingPatterns();
        placeDarkModule();
        addInformationReserveAreas();
        printDebugMatrix();
    }

    private void addInformationReserveAreas() {
        if (client.getVersion().version < 6) {
            for (int i = 0; i < 8; i++) {
                if (i != 6) {
                    matrix[i][8] = 3;
                    matrix[8][i] = 3;
                }

                matrix[8][width - i - 1] = 3;

                if (i != 7) {
                    matrix[width - i - 1][8] = 3;
                }
            }
            matrix[8][8] = 3;
        }
        else {
            for (int k = 0; k < 3; k++) {
                for (int i = matrix.length - 11, l = 0; l < 6; l++) {
                    matrix[l][i + k] = 3;
                    matrix[i + k][l] = 3;
                }
            }
        }
    }

    private void placeDarkModule() {
        int x = 8;
        int y = (4 * client.getVersion().version) + 9;
        matrix[y][x] = 9;
    }

    private void addFinderPatterns() {
        int actualPos = 0;
        for (int y = 0; y < 7; y++) {
            for (int x = 0; x < 7; x++) {
                matrix[y][x] = finderPattern[y][x];
            }
        }
        actualPos = width - 7;
        for (int y = actualPos, i = 0; y < width; y++, i++) {
            for (int x = 0, j = 0; x < 7; x++, j++) {
                matrix[y][x] = finderPattern[i][j];
            }
        }
        actualPos = width - 7;
        for (int y = 0, i = 0; y < 7; y++, i++) {
            for (int x = actualPos, j = 0; x < actualPos + 7; x++, j++) {
                matrix[y][x] = finderPattern[i][j];
            }
        }
    }

    private void addSeparators() {
        for (int i = 0; i < 8; i++) {
            matrix[i][7] = 2;
            matrix[7][i] = 2;

            matrix[i][width - 8] = 2;
            matrix[7][width - i - 1] = 2;

            matrix[width - 8][i] = 2;
            matrix[width - i - 1][7] = 2;
        }
    }

    private void addAlignmentPatterns() {
        int[] positions = client.getAlignmentPositionByVersion();
        for (int pos1 : positions) {

            for (int i = 0; i < positions.length; i++) {
                int pos2 = positions[i];
                if (isPosInFinderPattern(pos1, pos2)) continue;
                int x = pos1 - 2;
                int y = pos2 - 2;
                addAlignmentPatternAtPos(x, y);
            }
        }
    }

    private boolean isPosInFinderPattern(int x, int y) {
        if (x < 8 && y < 8) return true;
        if (x < 8 && y > width - 8) return true;
        if (y < 8 && x > width - 8) return true;
        return false;
    }

    private void addAlignmentPatternAtPos(int x, int y) {
        for (int i = 0; i < 5; i++) {
            for (int j = 0; j < 5; j++) {
                matrix[x + i][y + j] = alignmentPattern[i][j];
            }
        }
    }

    private void addTimingPatterns() {
        int statue = 8;
        for (int i = 7; i < width - 7; i++) {
            matrix[i][6] = statue;
            matrix[6][i] = statue;
            statue = (statue == 9) ? 8 : 9;
        }

    }

    private void placeBits() {
        bytes = client.getBytes() + getCorrectionStringFromNumbers();
        char[] bytesArray = bytes.toCharArray();

        boolean directionUp = true;
        int actual = 0;

//        System.out.println();
//        System.out.println(bytes);
//        System.out.println();

        for (int x = width - 1; x > 0; x -= 2) {
            if (directionUp) {
                int y = width - 1;
                while (directionUp && actual < bytesArray.length) {
                    for (int i = 0; i < 2; i++) {
                        if (y == -1) {
                            directionUp = false;
                            break;
                        }
                        int m = matrix[y][x - i];
                        if (m == -1) {
//                            System.out.println("actual: " + actual + " length: " + bytesArray.length);
//                            System.out.println("rest: " + bytes.substring(actual));
//                            int a = Integer.parseInt(String.valueOf(bytes.charAt(actual)));
                            int a = 0;
                            if (actual < bytesArray.length) {
                                a = Integer.parseInt(String.valueOf(bytesArray[actual]));
                            }
                            matrix[y][x - i] = a;
                            actual++;
//                            printDebugMatrix();
                        } else {
                            int[] tmp = getNextLocation(x, y, true);
                            if (tmp != null) {
                                y = tmp[0];
                                i = tmp[2] - 1;
                                if (tmp.length > 3) x -= tmp[3];
                                continue;
                            } else {
                                directionUp = false;
                                break;
                            }
                        }
                    }
                    y--;
                }
            } else {
                int y = 0;
                while (!directionUp && actual < bytesArray.length) {
                    if (y == width) {
                        y = width - 1;
                        directionUp = true;
                        break;
                    }
                    for (int i = 0; i < 2; i++) {
                        int m = matrix[y][x - i];
                        if (m == -1) {
//                            System.out.println("actual: " + actual + " length: " + bytesArray.length);
//                            System.out.println("rest: " + bytes.substring(actual));
//                            matrix[y][x - i] = Integer.parseInt(String.valueOf(bytes.charAt(actual)));
                            int a = 0;
                            if (actual < bytesArray.length) {
                                a = Integer.parseInt(String.valueOf(bytesArray[actual]));
                            }
                            matrix[y][x - i] = a;
//                            printDebugMatrix();
                            actual++;
                        } else {
                            int[] tmp = getNextLocation(x, y, false);
                            if (tmp != null) {
                                y = tmp[0];
                                i = tmp[2] - 1;
                                continue;
                            } else {
                                directionUp = true;
                                break;
                            }
                        }
                    }
                    y++;
                }
            }
        }
        printDebugMatrix();
//        System.out.println("actual: " + actual);
    }

    private String getCorrectionStringFromNumbers() {
        String result = new String();

        for (Integer i : correctionBits) {
            String next = String.format("%08d", Integer.valueOf(Integer.toBinaryString(i)));
            result += next;
        }

        return result;
    }

    private int[] getNextLocation(int posX, int posY, boolean up) {
        if (posY == 8 && posX == 8) {
            return new int[] { 9, 5, 2, 1 };
        }
        for (int y = posY; (up) ? y > 0 : y < width - 1;) {
            for (int x = 0; x < 2; x++) {
                int tmpX = posX - x;
                int m = matrix[y][tmpX];
                if (m == -1) return new int[] { y, tmpX, x };
            }
            y = (up) ? y - 1 : y + 1;
        }
        return null;
    }

    private int getFirstAvailableYPos(int x) {
        for (int y = 0; y < width - 1; y++) {
            if (matrix[y][x] == -1) return y;
        }
        return 0;
    }

    public void printDebugMatrix() {
        for (int i = 0; i < width; i++) {
            String line = "";
            for (int j = 0; j < width; j++) {
                if (matrix[i][j] != -1) {
                    line += matrix[i][j] + " ";
                } else {
                    line += "  ";
                }
            }
            System.out.println(line);
        }
        System.out.println("---------------------------------------------------------------");
    }

    private void printDebugMatrix(int[][] m) {
        for (int i = 0; i < width; i++) {
            String line = "";
            for (int j = 0; j < width; j++) {
                if (m[i][j] != -1) {
                    line += m[i][j] + " ";
                } else {
                    line += "  ";
                }
            }
            System.out.println(line);
        }
        System.out.println("---------------------------------------------------------------");
    }




}
