package com.company.QR;

import com.company.QRCodeClient;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

public class QRCodeVersion {
    private QRCodeClient client;
    private HashMap capabilites;
    private HashMap sizes;
    private HashMap alignmentPattersLocations;
    private HashMap maskInfoBits;

    public int version;

    public QRCodeVersion(QRCodeClient c) {
        capabilites = initCapabilites();
        sizes = initSizes();
        alignmentPattersLocations = initAlignPatternsLocations();
        client = c;

        initVersion();
    }

    private void initVersion() {
        int contentLenght = client.getMsg().length();
        int corLvl = client.getIntCorrectionlevel();

        //  protocena M a Q
        for (int i = 0; i < capabilites.size(); i++) {
            Block b = (Block) capabilites.get(i + 1);
            if (b.alphanumeric[corLvl] >= contentLenght) {
                version = i + 1;
                break;
            }
        }
    }

    public int[] generateVersionData(int mask) {
        if (version < 7) {
            initMaskInfoBits();
            String v = client.getStringCorrectionLevel();
            FormatStringInfoBlock info = (FormatStringInfoBlock) maskInfoBits.get(v);
            System.out.println("INFO BLOCK: " + info.get(mask));
            return arrayFromString(info.get(mask));
        } else {

            return null;
        }
    }

    private int[] arrayFromString(String input) {
        char[] n = input.toCharArray();
        int[] c = new int[input.length()];
        for (int i = 0; i < input.length(); i++) {
            try {
                c[i] = Integer.parseInt(String.valueOf(n[i]));
            } catch (Exception e) {}
        }
        return c;
    }

    private String stringFromArray(int[] input) {
        String s = "";
        for (int i = 0; i < input.length; i++) {
            s += input[i];
        }
        return s;
    }

    private int[] calculateCorrectionBits(int[] current) {
        int[] c = removeNullsAtStart(current);
        if (c.length == 12) {
            return c;
        }
        if (current.length < 12) return addNullAtStart(current);

        int[] pol = getPolNumbers(c.length);
        int[] xor = xorArrays(c, pol);

        return calculateCorrectionBits(xor);
    }

    private int[] xorArrays(int[] a1, int[] a2) {
        int[] a = new int[a1.length];
        for (int i = 0; i < a1.length; i++) {
            a[i] = a1[i] ^ a2[i];
        }
        return a;
    }

    private int[] addNullAtStart(int[] c) {
        String s = stringFromArray(c);
        char[] nulls = new char[10 - c.length];
        Arrays.fill(nulls, '0');
        return arrayFromString(new String(nulls) + s);
    }

    private int[] getPolNumbers(int lenght) {
        String p = "10100110111";
        for (int i = p.length(); i < lenght; i++) {
            p += "0";
        }
        return arrayFromString(p);
    }

    private int[] removeNullsAtStart(int[] nums) {
        String s = stringFromArray(nums);
        s = s.substring(s.indexOf("1"));
        return arrayFromString(s);
    }

    public int getVersion() { return version; }

    public int getSizeForVersion() {
        return (Integer) sizes.get(version);
    }

    private void initMaskInfoBits() {
        maskInfoBits = new HashMap();
        maskInfoBits.put("L", new FormatStringInfoBlock(
                "111011111000100",
                "111001011110011",
                "111110110101010",
                "111100010011101",
                "110011000101111",
                "110001100011000",
                "110110001000001",
                "110100101110110"));
        maskInfoBits.put("M", new FormatStringInfoBlock(
                "101010000010010",
                "101000100100101",
                "101111001111100",
                "101101101001011",
                "100010111111001",
                "100000011001110",
                "100111110010111",
                "100101010100000"));
        maskInfoBits.put("Q", new FormatStringInfoBlock(
                "011010101011111",
                "011000001101000",
                "011111100110001",
                "011101000000110",
                "010010010110100",
                "010000110000011",
                "010111011011010",
                "010101111101101"));
        maskInfoBits.put("H", new FormatStringInfoBlock(
                "001011010001001",
                "001001110111110",
                "001110011100111",
                "001100111010000",
                "000011101100010",
                "000001001010101",
                "000110100001100",
                "000100000111011"));

    }

    private HashMap initAlignPatternsLocations() {
        HashMap map = new HashMap();
        map.put(1, new int[] {});
        map.put(2, new int[] {6, 18});
        map.put(3, new int[] {6, 22});
        map.put(4, new int[] {6, 26});
        map.put(5, new int[] {6, 30});
        map.put(6, new int[] {6, 34});
        map.put(7, new int[] {6, 22, 38});
        map.put(8, new int[] {6, 24, 42});
        map.put(9, new int[] {6, 26, 46});
        map.put(10, new int[] {6, 28, 50});
        map.put(11, new int[] {6, 30, 54});
        map.put(12, new int[] {6, 32, 58});
        map.put(13, new int[] {6, 34, 62});
        map.put(14, new int[] {6, 26, 26, 66});
        map.put(15, new int[] {6, 26, 48, 70});
        map.put(16, new int[] {6, 26, 50, 74});
        map.put(17, new int[] {6, 30, 54, 78});
        map.put(18, new int[] {6, 30, 56, 82});
        map.put(19, new int[] {6, 30, 58, 86});
        map.put(20, new int[] {6, 34, 62, 90});
        map.put(21, new int[] {6, 28, 50, 72, 94});
        map.put(22, new int[] {6, 26, 50, 74, 98});
        map.put(23, new int[] {6, 30, 54, 78, 102});
        map.put(24, new int[] {6, 28, 54, 80, 106});
        map.put(25, new int[] {6, 32, 58, 84, 110});
        map.put(26, new int[] {6, 30, 58, 86, 114});
        map.put(27, new int[] {6, 34, 63, 90, 118});
        map.put(28, new int[] {6, 26, 50, 74, 98, 112});
        map.put(29, new int[] {6, 30, 54, 78, 102, 126});
        map.put(30, new int[] {6, 26, 52, 78, 104, 130});
        map.put(31, new int[] {6, 30, 56, 82, 108, 134});
        map.put(32, new int[] {6, 34, 60, 86, 112, 138});
        map.put(33, new int[] {6, 30, 58, 86, 114, 142});
        map.put(34, new int[] {6, 34, 62, 90, 118, 146});
        map.put(35, new int[] {6, 30, 54, 78, 102, 126, 150});
        map.put(36, new int[] {6, 24, 50, 67, 102, 128, 154});
        map.put(37, new int[] {6, 28, 54, 80, 106, 143, 158});
        map.put(38, new int[] {6, 32, 58, 84, 110, 136, 162});
        map.put(39, new int[] {6, 26, 54, 82, 110, 138, 166});
        map.put(40, new int[] {6, 30, 58, 86, 114, 142, 170});

        return map;
    }

    public int[] getAlignmentPositionByVersion() {
        return (int[]) alignmentPattersLocations.get(version);
    }

    private HashMap initSizes() {
        HashMap map = new HashMap();
        for (int i = 1, w = 21; i <= 40; i++, w += 4) {
            map.put(i, w);
        }
        return map;
    }

    public int getCapability(int version, int ecc) {
        Block b = (Block) capabilites.get(version);
        return b.alphanumeric[ecc];
    }

    private HashMap initCapabilites() {
        HashMap map = new HashMap();
        map.put(1, new Block(25, 20, 16, 10));
        map.put(2, new Block(47, 38, 29, 20));
        map.put(3, new Block(77, 61, 47, 35));
        map.put(4, new Block(114, 90, 67, 50));
        map.put(5, new Block(154, 122, 87, 64));
        map.put(6, new Block(195, 154, 108, 84));
        map.put(7, new Block(224, 178, 125, 93));
        map.put(8, new Block(279, 221, 157, 122));
        map.put(9, new Block(335, 262, 189, 143));
        map.put(10, new Block(395, 311, 221, 174));
        map.put(11, new Block(468, 366, 259, 200));
        map.put(12, new Block(535, 419, 296, 227));
        map.put(13, new Block(619, 483, 352, 259));
        map.put(14, new Block(667, 528, 376, 283));
        map.put(15, new Block(758, 600, 426, 321));
        map.put(16, new Block(854, 656, 470, 365));
        map.put(17, new Block(938, 734, 531, 408));
        map.put(18, new Block(1046, 816, 574, 452));
        map.put(19, new Block(1153, 909, 744, 493));
        map.put(20, new Block(1249, 970, 702, 557));
        map.put(21, new Block(1352, 1035, 742, 587));
        map.put(22, new Block(1460, 1134, 823, 640));
        map.put(23, new Block(1588, 1248, 890, 672));
        map.put(24, new Block(1704, 1326, 963, 744));
        map.put(25, new Block(1853, 1451, 1041, 779));
        map.put(26, new Block(1990, 1542, 1094, 864));
        map.put(27, new Block(2132, 1637, 1172, 910));
        map.put(28, new Block(2223, 1732, 1263, 958));
        map.put(29, new Block(2369, 1839, 1322, 1016));
        map.put(30, new Block(2520, 1994, 1429, 1080));
        map.put(31, new Block(2677, 2113, 1499, 1150));
        map.put(32, new Block(2840, 2238, 1618, 1226));
        map.put(33, new Block(3009, 2369, 1700, 1307));
        map.put(34, new Block(3183, 2506, 1787, 1394));
        map.put(35, new Block(3351, 2632, 1867, 1431));
        map.put(36, new Block(3537, 2780, 1966, 1530));
        map.put(37, new Block(3729, 2894, 2071, 1591));
        map.put(38, new Block(3927, 3054, 2181, 1658));
        map.put(39, new Block(4087, 3220, 2298, 1774));
        map.put(40, new Block(4296, 3391, 2420, 1852));

        return map;
    }

    class FormatStringInfoBlock {
        String[] bits;

        public FormatStringInfoBlock(String s0,
                                     String s1,
                                     String s2,
                                     String s3,
                                     String s4,
                                     String s5,
                                     String s6,
                                     String s7) {
            bits = new String[8];
            bits[0] = s0;
            bits[1] = s1;
            bits[2] = s2;
            bits[3] = s3;
            bits[4] = s4;
            bits[5] = s5;
            bits[6] = s6;
            bits[7] = s7;
        }

        public String get(int id) {
            return bits[id];
        }

        public void add(int id, String m) {
            bits[id] = m;
        }
    }

    class Block {
        public Block(int L, int M, int Q, int H) {
            alphanumeric = new int[4];
            alphanumeric[0] = L;
            alphanumeric[1] = M;
            alphanumeric[2] = Q;
            alphanumeric[3] = H;
        }

        public int[] alphanumeric;
    }

}
