package com.company.QR;

import com.company.QRCodeClient;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

public class QRTextCoder {

    private QRCodeClient client;

    private String _message;
    private int charCount;

    private String bytes;
    private String modeIndicator;
    private String characterCountIndicator;
    private String terminator = "";
    private int totalCapacity;

    private String padBytes1 = "11101100";
    private String padBytes2 = "00010001";

    private int version;


    public QRTextCoder(QRCodeClient c) {
        client = c;
        _message = client.getMsg().toUpperCase();
        charCount = _message.length();
        bytes = "";

        process();
    }

    public void process() {
        initBytesHead();
        System.out.println("-----------HeaderBites-------------");
        System.out.println("ModeIndicator: \t\t" + modeIndicator);
        System.out.println("CharacterCountIndicator: \t" + characterCountIndicator);
        System.out.println("-----------------------------------");

        encode();
        System.out.println("-----------EncodedMessage----------");
        System.out.println("bytes: " + bytes);
        System.out.println("-----------------------------------");

        addTerminator();
        System.out.println("-----------AddedTerminator---------");
        System.out.println("totalCapacity: " + totalCapacity);
        System.out.println("terminator:    " + terminator);
        System.out.println("-----------------------------------");

        makeBytesMultipleOf8();
        System.out.println("-----------MultiplyOf8-------------");
        System.out.println("bytes: " + bytes);
        System.out.println("-----------------------------------");

        addPadBytes();
        System.out.println("-----------PadBytes----------------");
        System.out.println("bytes: " + bytes);
        System.out.println("-----------------------------------");

    }

    private void addPadBytes() {
        int statue = 1;
        while (bytes.length() != totalCapacity && bytes.length() < totalCapacity) {
            if (statue == 1) {
                bytes += padBytes1;
                statue = 2;
            } else {
                bytes += padBytes2;
                statue = 1;
            }
        }
    }

    private void addTerminator() {
        int tmp = client.getTotalCapability();
        totalCapacity = tmp * 8;
        int msgLenght = _message.length();

        if (totalCapacity - msgLenght >= 4) {
            terminator = "0000";
        } else {
            for (int i = 0; i < totalCapacity - msgLenght; i++) {
                terminator += "0";
            }
        }
        bytes += terminator;
    }

    private void encode() {
        int maxlenght = 0;
        ArrayList<String> encoding = new ArrayList<String>();
        int temp = 0;
        String last = "";
        for (int i = 1; i <= charCount; i++) {
            if (i % 2 != 0) {
                if (i + 1 > charCount) {
                    temp += getCharCode(_message.charAt(i - 1));
                    last = getBinaryValueWithoutNulls(temp, true);
//                    bytes += getBinaryValue(temp, true);
                } else {
                    temp += 45 * getCharCode(_message.charAt(i - 1));
                }
            } else {
                temp += getCharCode(_message.charAt(i - 1));
//                System.out.println(getBinaryValue(temp, false));
                String s = getBinaryValueWithoutNulls(temp, false);
                if (s.length() > maxlenght) maxlenght = s.length();
                encoding.add(s);
//                bytes += getBinaryValue(temp, false);
                temp = 0;
            }
        }

        for (String s : encoding) {
            s = addNullsToCount(maxlenght, s);
            System.out.println(s);
            bytes += s;
        }
        bytes += last;
    }

    private void initBytesHead() {
        modeIndicator = "0010";
//        modeIndicator = String.format("%4s", Integer.toBinaryString(client.getMode() & 0xFF)).replace(' ', '0');
        version = getVersion();
        char[] countIndicator = null;

        String binaryCount = getBinaryValue(_message.length());
        switch (version) {
            case 1:
                countIndicator = new char[9 - binaryCount.length()];
                break;
            case 2:
                countIndicator = new char[11 - binaryCount.length()];
                break;
            case 3:
                countIndicator = new char[13 - binaryCount.length()];
                break;
        }
        Arrays.fill(countIndicator, '0');

        characterCountIndicator = new String(countIndicator) + binaryCount;

        bytes = modeIndicator + characterCountIndicator + bytes;
    }

    private int getVersion() {
        String bin = getBinaryValue(_message.length());
        if (bin.length() <= 9) {
            return 1;
        } else if (bin.length() <= 29) {
            return 2;
        } else {
            return 3;
        }
    }

    private void makeBytesMultipleOf8() {
        while (bytes.length() % 8 != 0) {
            bytes += "0";
        }
    }

    public String getBytes() {
        return bytes;
    }

    private void printBytes() {
        System.out.println("----------------------------------");
        System.out.println(bytes);
        System.out.println("----------------------------------");
    }

    private String getBinaryValue(int value) {
        return Integer.toBinaryString(value);
    }

    private String getBinaryValue(int value, Boolean last) {
//        System.out.println("Binary from: " + value);
        if (!last) return addNullsToLength(Integer.toBinaryString(value));
        else return "00" + Integer.toBinaryString(value);
    }

    private String addNullsToCount(int c, String input) {
        int count = c - input.length();
        if (count < 0) count *= -1;
        char[] nulls = new char[count];
        Arrays.fill(nulls, '0');
        return new String(nulls) + input;
    }

    private String getBinaryValueWithoutNulls(int value, Boolean last) {
        if (!last) return Integer.toBinaryString(value);
        else return addNullsToCount(6, Integer.toBinaryString(value));
    }

    private String addNullsToLength(String input) {
        int count = _message.length() - input.length();
        if (count < 0) return input;
//        if (count < 0) count *= -1;
        char[] nulls = new char[count];
        Arrays.fill(nulls, '0');
        return new String(nulls) + input;
    }

    private int getCharCode(char c) {
//        System.out.println("char: " + c + " " + client.getAlphaNumericTable().get(c));
        return (Integer) client.getAlphaNumericTable().get(c);
    }

    /*
    - spocitat pocet znaku zpravy (napr. 11 znaku)
    - typ + pocet znaku (pridany zleva nuly podle toho, jakou velikost chceme)
        1 - 9   = 10/9/8/8 znaku
        10 - 26 = 12/11/16/10 znaku
        27 - 40 = 14/13/16/12 znaku
                0010 000001011 (alphanumeric, 11 char in 1-9 velikosti
    - alphanumeric
        - rozdelit na dvojice
        - prvni znak vynasobit (45 * ascii kod znaku) + druhy ascii znak
        - vysledek prevest do pocet znaku-binaru
        - pridat kontrolni bity
        - prevest do 8 bit slov
    */

    /*
    0001 - numeric
    0010 - alphanumeric
    0100 - byte
    1000 - kanji
    */
}
